using Microsoft.EntityFrameworkCore;
using Microsoft.Extensions.Diagnostics.HealthChecks;
using Microsoft.OpenApi.Models;
using Minio;
using Minio.AspNetCore;
using Minio.AspNetCore.HealthChecks;
using PaydayBackend;
using PaydayBackend.Services;

var builder = WebApplication.CreateBuilder(args);

string GetEnv(string envName, string settingsName = "")
{
    string? dbConString = builder.Configuration.GetConnectionString(settingsName) ?? Environment.GetEnvironmentVariable(envName);
    if (!string.IsNullOrEmpty(dbConString))
        return dbConString;

    Console.WriteLine($"Environment variable {envName} not found.");
    return String.Empty;
}

builder.Services.AddControllers();
// Learn more about configuring Swagger/OpenAPI at https://aka.ms/aspnetcore/swashbuckle
builder.Services.AddEndpointsApiExplorer();
builder.Services.AddSwaggerGen(c =>
{
    c.IncludeXmlComments(Path.Combine(AppContext.BaseDirectory, "ApiDocumentation.xml"));
    c.SwaggerDoc("v1", new OpenApiInfo { Title = "Backend", Version = "v1" });
});

// Database
string? dbConString = GetEnv("CONNECTION_STRING", "DefaultConnection");

builder.Services.AddDbContext<DatabaseContext>(options =>
    { options.UseNpgsql(dbConString); });

// HealthChecks
builder.Services.AddHealthChecks()
    .AddNpgSql(dbConString)
    .AddMinio(failureStatus: HealthStatus.Degraded, factory: sp => sp.GetRequiredService<MinioClient>());

// Services
builder.Services.AddSingleton<IStorageService, StorageService>();
builder.Services.AddScoped<IAdminService, AdminService>();
builder.Services.AddScoped<IUserService, UserService>();
builder.Services.AddScoped<IPublicService, PublicService>();

// Minio
builder.Services.AddMinio(options =>
{
    options.Endpoint = GetEnv("S3_ENDPOINT", "S3Endpoint");
    options.AccessKey = GetEnv("S3_ACCESS_KEY", "S3AccessKey");
    options.SecretKey = GetEnv("S3_SECRET_KEY", "S3SecretKey");
    
    options.ConfigureClient(client =>
    {
        client.WithSSL(false);
    });
});

var app = builder.Build();

// Configure the HTTP request pipeline.
// if (app.Environment.IsDevelopment())
// {
    app.UseSwagger();
    app.UseSwaggerUI();
// }

app.MapHealthChecks("/health");
// app.UseHttpsRedirection();

// app.UseAuthorization();

app.MapControllers();

app.Run();