﻿using SixLabors.ImageSharp.Formats.Jpeg;

namespace PaydayBackend.Utils;

public static class FileHelper
{
     public static string GetMimeType(string filename)
    {
        var extension = filename.Split(".").Last();
        switch (extension)
        {
            case "jpg":
            case "jpeg":
                return "image/jpeg";
            case "png":
                return "image/png";
            case "csv":
                return "text/csv";
            case "pdf":
                return "application/pdf";
            case "html":
                return "text/html";
            default:
                throw new ArgumentException($"Unsupported file type, file: {filename}");
        }
    }

    // Функция валидации размера файла
    public static string ValidateMaxFileSize(IFormFile file, int maxSizeMb, string[] allowedExtensions)
    {
        if (file.Length > 0)
        {
            if (file.Length > maxSizeMb * 1024 * 1024)
                return $"File need to be less than {maxSizeMb}MB.";
            if (!allowedExtensions.Contains(Path.GetExtension(file.FileName).ToLower()))
                return "Invalid file extension.";

            return "OK";
        }

        return "Invalid file.";
    }

    // Функция валидации изображения
    public static async Task<string> ValidateImage(MemoryStream fileStream, string[]? allowedExtensions, int minWidth = 0,
        int minHeight = 0, int maxWidth = 0, int maxHeight = 0)
    {
        if (allowedExtensions == null)
            allowedExtensions = new[] {"JPG", "JPEG", "PNG"};
        
        try
        {
            var imageInfo = await Image.DetectFormatAsync(fileStream);
            if (!allowedExtensions.Contains(imageInfo.Name))
                return "Image type is not" + allowedExtensions.Aggregate("",
                    (current, next) => current + " " + next) + ".";
            
            using var image = await Image.LoadAsync(fileStream);
            if (image.Width < minWidth || image.Height < minHeight)
                return $"Image size must be at least {minWidth}x{minHeight}.";

            if (maxHeight != 0 && maxWidth != 0)
                if (image.Width > maxWidth || image.Height > maxHeight)
                    return $"Image size must be less than {maxWidth}x{maxHeight}.";

            return "OK";
        }
        catch (ImageFormatException e)
        {
            Console.WriteLine(e.Message);
            return "Invalid image type.";
        }
    }

    // Кропает изображение по центру, в зависимости от того, какая сторона меньше
    public static async Task CropImage(MemoryStream fileStream)
    {
        fileStream.Position = 0;
        using var image = await Image.LoadAsync(fileStream);
        Console.WriteLine(image.Height/2 - image.Width/2 + " " + image.Width + " " + image.Height);
        if (image.Height > image.Width)
            image.Mutate(i => i.Crop(new Rectangle(0, image.Height/2 - image.Width/2, image.Width, image.Width)));
        else
            image.Mutate(i => i.Crop(new Rectangle(image.Width/2 - image.Height/2, 0, image.Height, image.Height)));
       
        await fileStream.FlushAsync();
        fileStream.Position = 0;
        await image.SaveAsync(fileStream, new JpegEncoder());
    }
}